@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 """
+Holds the code for cleaning out unwanted tags from the lxml
+dom xpath.
 """
 from .utils import ReplaceSequence
 
@@ -7,12 +9,14 @@ class DocumentCleaner(object):
 
     def __init__(self, config):
         self.config = config
-        # parser
         self.parser = self.config.get_parser()
+
         self.remove_nodes_re = (
-        "^side$|combx|retweet|mediaarticlerelated|menucontainer|navbar"
+        "^side$|combx|retweet|mediaarticlerelated|menucontainer|"
+        "navbar|storytopbar-bucket|utility-bar|inline-share-tools"
         "|comment|PopularQuestions|contact|foot|footer|Footer|footnote"
-        "|cnn_strycaptiontxt|links|meta$|scroll|shoutbox|sponsor"
+        "|cnn_strycaptiontxt|cnn_html_slideshow|cnn_strylftcntnt"
+        "|links|meta$|shoutbox|sponsor"
         "|tags|socialnetworking|socialNetworking|cnnStryHghLght"
         "|cnn_stryspcvbx|^inset$|pagetools|post-attributes"
         "|welcome_form|contentTools2|the_answers"
@@ -41,6 +45,7 @@ class DocumentCleaner(object):
         """
         """
         doc_to_clean = article.doc
+        doc_to_clean = self.clean_body_classes(doc_to_clean)
         doc_to_clean = self.clean_article_tags(doc_to_clean)
         doc_to_clean = self.clean_em_tags(doc_to_clean)
         doc_to_clean = self.remove_drop_caps(doc_to_clean)
@@ -57,6 +62,15 @@ class DocumentCleaner(object):
         doc_to_clean = self.div_to_para(doc_to_clean, 'span')
         return doc_to_clean
 
+    def clean_body_classes(self, doc):
+        # we don't need body classes
+        # in case it matches an unwanted class all the document
+        # will be empty
+        elements = self.parser.getElementsByTag(doc, tag="body")
+        if elements:
+            self.parser.delAttribute(elements[0], attr="class")
+        return doc
+
     def clean_article_tags(self, doc):
         articles = self.parser.getElementsByTag(doc, tag='article')
         for article in articles:
@@ -124,7 +138,7 @@ class DocumentCleaner(object):
         return doc
 
     def clean_para_spans(self, doc):
-        spans = self.parser.css_select(doc, 'p > span')
+        spans = self.parser.css_select(doc, 'p span')
         for item in spans:
             self.parser.drop_tag(item)
         return doc
