@@ -1,11 +1,20 @@
 # -*- coding: utf-8 -*-
+"""
+Newspaper uses a lot of python-goose's extraction code. View their
+license here: https://github.com/codelucas/newspaper/blob/master/GOOSE-LICENSE.txt
 
+Keep all html page extraction code within this file. PLEASE abstract any
+lxml or soup parsing mechanisms in the parsers.py file!
 """
-"""
+__title__ = 'newspaper'
+__author__ = 'Lucas Ou-Yang'
+__license__ = 'MIT'
+__copyright__ = 'Copyright 2014, Lucas Ou-Yang'
 
 import re
 import copy
 import urlparse
+from collections import defaultdict
 
 from .packages.tldextract import tldextract
 from .utils import (
@@ -115,7 +124,7 @@ class ContentExtractor(object):
         VALS = ['author', 'byline']
         matches = []
         _authors, authors = [], []
-        doc = article.doc
+        doc = article.clean_doc
         html = article.html
 
         for attr in ATTRS:
@@ -164,7 +173,7 @@ class ContentExtractor(object):
         Fetch the article title and analyze it.
         """
         title = ''
-        doc = article.doc
+        doc = article.clean_doc
 
         title_element = self.parser.getElementsByTag(doc, tag='title')
         # no title found
@@ -217,6 +226,29 @@ class ContentExtractor(object):
         title = title_pieces[large_text_index]
         return TITLE_REPLACEMENTS.replaceAll(title).strip()
 
+    def get_feed_urls(self, source_or_category):
+        """
+        Returns list of feed urls on a source or category object.
+        """
+        # This feels really weird..., needs more refactoring
+        is_source = source_or_category.__class__.__name__ == 'Source'
+
+        if not is_source: # recursively call method with Source's categories
+            feed_urls = []
+            for category in source_or_category.categories:
+                feed_urls.extend(self.get_feed_urls(category))
+            feed_urls = feed_urls[:50]
+            feed_urls = [prepare_url(f, source.url) for f in feed_urls]
+
+            feed_urls = list(set(feed_urls))
+            return feed_urls
+
+        doc = source_or_category.doc # it's a Category
+        kwargs = {'attr': 'type', 'value': 'application/rss+xml'}
+        feed_elements = self.parser.getElementsByTag(doc, **kwargs)
+        feed_urls = [e.get('href') for e in feed_elements if e.get('href')]
+        return feed_urls
+
     def get_favicon(self, article):
         """
         Extract the favicon from a website
@@ -225,7 +257,7 @@ class ContentExtractor(object):
         <link rel="icon" type="image/png" href="favicon.png" />
         """
         kwargs = {'tag': 'link', 'attr': 'rel', 'value': 'icon'}
-        meta = self.parser.getElementsByTag(article.doc, **kwargs)
+        meta = self.parser.getElementsByTag(article.clean_doc, **kwargs)
         if meta:
             favicon = self.parser.getAttribute(meta[0], 'href')
             return favicon
@@ -236,7 +268,7 @@ class ContentExtractor(object):
         Extract content language from meta.
         """
         # we have a lang attribute in html
-        attr = self.parser.getAttribute(article.doc, attr='lang')
+        attr = self.parser.getAttribute(article.clean_doc, attr='lang')
         if attr is None:
             # look up for a Content-Language in meta
             items = [
@@ -244,7 +276,7 @@ class ContentExtractor(object):
                 {'tag': 'meta', 'attr': 'name', 'value': 'lang'}
             ]
             for item in items:
-                meta = self.parser.getElementsByTag(article.doc, **item)
+                meta = self.parser.getElementsByTag(article.clean_doc, **item)
                 if meta:
                     attr = self.parser.getAttribute(meta[0], attr='content')
                     break
@@ -258,6 +290,10 @@ class ContentExtractor(object):
     def get_meta_content(self, doc, metaName):
         """
         Extract a given meta content form document.
+        Example metaNames:
+            "meta[name=description]"
+            "meta[name=keywords]"
+            "meta[property=og:type]"
         """
         meta = self.parser.css_select(doc, metaName)
         content = None
@@ -270,24 +306,102 @@ class ContentExtractor(object):
 
         return ''
 
-    def get_meta_description(self, article):
+    def get_meta_img_url(self, article):
+        """
+        Returns the 'top img' as specified by the website.
+        """
+        top_meta_image, try_one, try_two, try_three, try_four = [None] * 5
+        doc = article.clean_doc
+
+        try_one = self.get_meta_content(doc, 'meta[property="og:image"]')
+        if try_one is None:
+            link_icon_kwargs = {'tag': 'link', 'attr': 'rel', 'value': 'icon'}
+            elems = self.parser.getElementsByTag(doc, **link_icon_kwargs)
+            try_two = elems[0].get('href') if elems else None
+
+        if try_two is None:
+            link_img_src_kwargs = {'tag': 'link', 'attr': 'rel', 'value': 'img_src'}
+            elems = self.parser.getElementsByTag(doc, **link_img_src_kwargs)
+            try_three = elems[0].get('href') if elems else None
+
+        if try_three is None:
+            try_four = self.get_meta_content(doc, 'meta[name="og:image"]')
+
+        top_meta_image = try_one or try_two or try_three or try_four # :)
+
+        return urlparse.urljoin(article.url, top_meta_image)
+
+    def get_meta_type(self, article):
+        """
+        Returns meta type of article, open graph protocol.
+        """
+        return self.get_meta_content(article.clean_doc, 'meta[property="og:type"]')
+
+    def get_meta_description(self, article_or_source):
         """
         If the article has meta description set in the source, use that.
         """
-        return self.get_meta_content(article.doc, "meta[name=description]")
+        # Since <source> objects use this particular method and sources don't
+        # have a 'clean_doc' we just use doc
+        try: # "easier to ask for forgiveness than permission"
+            doc = article_or_source.clean_doc
+        except:
+            doc = article_or_source.doc
+        return self.get_meta_content(doc, "meta[name=description]")
 
     def get_meta_keywords(self, article):
         """
         If the article has meta keywords set in the source, use that.
         """
-        return self.get_meta_content(article.doc, "meta[name=keywords]")
+        return self.get_meta_content(article.clean_doc, "meta[name=keywords]")
+
+    def get_meta_data(self, article):
+        data = defaultdict(dict)
+        props = self.parser.css_select(article.clean_doc, 'meta')
+
+        for prop in props:
+            key = prop.attrib.get('property')
+            if not key:
+                key = prop.attrib.get('name')
+
+            if not key:
+                continue
+
+            key = key.split(':')
+
+            value = prop.attrib.get('content')
+            if not value:
+                value = prop.attrib.get('value')
+
+            if not value:
+                continue
+
+            value = value.strip()
+
+            if value.isdigit():
+                value = int(value)
+
+            ref = data[key.pop(0)]
+
+            for idx, part in enumerate(key):
+                if not key[idx:-1]:  # no next values
+                    ref[part] = value
+                    break
+                if not ref.get(part):
+                    ref[part] = dict()
+                else:
+                    if isinstance(ref.get(part), basestring):
+                        ref[part] = {'url': ref[part]}
+                ref = ref[part]
+
+        return data
 
     def get_canonical_link(self, article):
         """
         If the article has meta canonical link set in the url.
         """
         kwargs = {'tag': 'link', 'attr': 'rel', 'value': 'canonical'}
-        meta = self.parser.getElementsByTag(article.doc, **kwargs)
+        meta = self.parser.getElementsByTag(article.clean_doc, **kwargs)
         if meta is not None and len(meta) > 0:
             href = self.parser.getAttribute(meta[0], 'href')
             if href:
@@ -300,22 +414,74 @@ class ContentExtractor(object):
                 return href
         return u''
 
-    def get_img_urls(self, article):
+    def get_img_urls(self, article, use_top_node=False):
         """
         Return all of the images on an html page, lxml root.
         """
-        doc = article.raw_doc
-        urls = self.parser.get_img_urls(doc)
-        img_links = [ urlparse.urljoin(article.url, url) for url in urls ]
+        doc = article.clean_top_node if use_top_node else article.clean_doc
 
+        img_kwargs = {'tag': 'img'}
+        img_tags = self.parser.getElementsByTag(doc, **img_kwargs)
+        urls = [img_tag.get('src') for img_tag in img_tags if img_tag.get('src')]
+        img_links = set([ urlparse.urljoin(article.url, url) for url in urls ])
+
+        if article.meta_img:
+            img_links.add(article.meta_img)
         return img_links
 
-    def get_top_img_url(self, article):
+    def get_first_img_url(self, article):
         """
+        Retrieves the first image in the 'top_node'
+        The top node is essentially the HTML
+        markdown where the main article lies and the first image
+        in that area is probably signifigcant.
         """
-        # !important, we must use raw_doc because at this point doc has been cleaned
-        doc = article.raw_doc
-        return self.parser.get_top_img_url(doc)
+        node_images = self.get_img_urls(article, use_top_node=True)
+        node_images = list(node_images)
+        if node_images:
+            return urlparse.urljoin(article.url, node_images[0])
+        return u''
+
+    def _get_urls(self, doc, titles):
+        """
+        Return a list of urls or a list of (url, title_text) tuples if specified.
+        """
+        if doc is None:
+            return []
+
+        a_kwargs = {'tag': 'a'}
+        a_tags = self.parser.getElementsByTag(doc, **a_kwargs) # doc.xpath('//a')
+
+        # TODO this should be refactored! We should have a seperate method which
+        # siphones the titles our of a list of <a> tags.
+        if titles:
+            return [ (a.get('href'), a.text) for a in a_tags if a.get('href') ]
+
+        return [ a.get('href') for a in a_tags if a.get('href') ]
+
+    def get_urls(self, doc_or_html, titles=False, regex=False):
+        """
+        doc_or_htmls html page or doc and returns list of urls, the regex
+        flag indicates we don't parse via lxml and just search the html.
+        """
+        if doc_or_html is None:
+            log.critical('Must extract urls from either html, text or doc!')
+            return []
+
+        # If we are extracting from raw text
+        if regex:
+            doc_or_html = re.sub('<[^<]+?>', ' ', doc_or_html)
+            doc_or_html = re.findall('http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+', doc_or_html)
+            doc_or_html = [i.strip() for i in doc_or_html]
+            return doc_or_html or []
+
+        # If the doc_or_html is html, parse it into a root
+        if isinstance(doc_or_html, str) or isinstance(doc_or_html, unicode):
+            doc = self.parser.fromstring(doc_or_html)
+        else:
+            doc = doc_or_html
+
+        return self._get_urls(doc, titles)
 
     def get_category_urls(self, source, source_url=None, page_urls=None):
         """
@@ -326,7 +492,7 @@ class ContentExtractor(object):
         """
 
         source_url = source.url if not source_url else source_url
-        page_urls = self.parser.get_urls(source.doc) if not page_urls else page_urls
+        page_urls = self.get_urls(source.doc) if not page_urls else page_urls
         valid_categories = []
         for p_url in page_urls:
             scheme = get_scheme(p_url, allow_fragments=False)
@@ -439,24 +605,8 @@ class ContentExtractor(object):
         category_urls = [c for c in category_urls if c is not None]
         return category_urls
 
-    def get_feed_urls(self, source):
-        """
-        Requires: List of category lxml roots, two types of anchors: categories
-        and feeds (rss). we extract category urls first and then feeds.
-        """
-        feed_urls = []
-        for category in source.categories:
-            root = category.doc
-            feed_urls.extend(self.parser.get_feed_urls(root))
-
-        feed_urls = feed_urls[:50]
-        feed_urls = [ prepare_url(f, source.url) for f in feed_urls ]
-
-        feeds = list(set(feed_urls))
-        return feeds
-
     def extract_tags(self, article):
-        node = article.doc
+        node = article.clean_doc
 
         # node doesn't have chidren
         if len(list(node)) == 0:
@@ -478,6 +628,11 @@ class ContentExtractor(object):
 
     def calculate_best_node(self, article):
         doc = article.doc
+
+        # if article.config.hints: TODO: Maybe a future release
+        #    rules_kw = article.config.hints.get(article.source_url)
+        #    doc = article.parser.getElementsByTag(doc, **rules_kw)[0]
+
         top_node = None
         nodes_to_check = self.nodes_to_check(doc)
 
@@ -510,7 +665,7 @@ class ContentExtractor(object):
                 if (nodes_number - i) <= bottom_negativescore_nodes:
                     booster = float(bottom_negativescore_nodes - (nodes_number - i))
                     boost_score = float(-pow(booster, float(2)))
-                    negscore = -abs(boost_score) + negative_scoring
+                    negscore = abs(boost_score) + negative_scoring
                     if negscore > 40:
                         boost_score = float(5)
 
