@@ -1,6 +1,12 @@
 # -*- coding: utf-8 -*-
 """
+Stopword extraction and stopword classes.
 """
+__title__ = 'newspaper'
+__author__ = 'Lucas Ou-Yang'
+__license__ = 'MIT'
+__copyright__ = 'Copyright 2014, Lucas Ou-Yang'
+
 import os
 import re
 import string
@@ -65,11 +71,16 @@ class StopWords(object):
     def remove_punctuation(self, content):
         # code taken form
         # http://stackoverflow.com/questions/265960/best-way-to-strip-punctuation-from-a-string-in-python
-        if isinstance(content, unicode):
+        content_is_unicode = isinstance(content, unicode)
+        if content_is_unicode:
             content = content.encode('utf-8')
-        return content.translate(self.TRANS_TABLE, string.punctuation)
+        stripped_input = content.translate(self.TRANS_TABLE, string.punctuation)
 
-    def candiate_words(self, stripped_input):
+        if content_is_unicode:
+            return stripped_input.decode('utf-8')
+        return stripped_input
+
+    def candidate_words(self, stripped_input):
         return stripped_input.split(' ')
 
     def get_stopword_count(self, content):
@@ -77,10 +88,10 @@ class StopWords(object):
             return WordStats()
         ws = WordStats()
         stripped_input = self.remove_punctuation(content)
-        candiate_words = self.candiate_words(stripped_input)
+        candidate_words = self.candidate_words(stripped_input)
         overlapping_stopwords = []
         c = 0
-        for w in candiate_words:
+        for w in candidate_words:
             c += 1
             if w.lower() in self.STOP_WORDS:
                 overlapping_stopwords.append(w.lower())
@@ -98,7 +109,7 @@ class StopWordsChinese(StopWords):
     def __init__(self, language='zh'):
         super(StopWordsChinese, self).__init__(language='zh')
 
-    def candiate_words(self, stripped_input):
+    def candidate_words(self, stripped_input):
         # jieba builds a tree that takes a while. avoid building
         # this tree if we don't use the chinese language
         from .packages import jieba
@@ -116,7 +127,7 @@ class StopWordsArabic(StopWords):
     def remove_punctuation(self, content):
         return content
 
-    def candiate_words(self, stripped_input):
+    def candidate_words(self, stripped_input):
         import nltk
         s = nltk.stem.isri.ISRIStemmer()
         words = []
@@ -137,10 +148,10 @@ class StopWordsKorean(StopWords):
             return WordStats()
         ws = WordStats()
         stripped_input = self.remove_punctuation(content)
-        candiate_words = self.candiate_words(stripped_input)
+        candidate_words = self.candidate_words(stripped_input)
         overlapping_stopwords = []
         c = 0
-        for w in candiate_words:
+        for w in candidate_words:
             c += 1
             for stop_word in self.STOP_WORDS:
                 overlapping_stopwords.append(stop_word)
