@@ -3,6 +3,11 @@
 Newspaper treats urls for news articles as critical components.
 Hence, we have an entire module dedicated to them.
 """
+__title__ = 'newspaper'
+__author__ = 'Lucas Ou-Yang'
+__license__ = 'MIT'
+__copyright__ = 'Copyright 2014, Lucas Ou-Yang'
+
 import logging
 import re
 
@@ -16,6 +21,11 @@ log = logging.getLogger(__name__)
 
 MAX_FILE_MEMO = 20000
 
+DATE_REGEX = r'([\./\-_]{0,1}(19|20)\d{2})[\./\-_]{0,1}(([0-3]{0,1}[0-9][\./\-_])|(\w{3,5}[\./\-_]))([0-3]{0,1}[0-9][\./\-]{0,1})?'
+
+ALLOWED_TYPES = ['html', 'htm', 'md', 'rst', 'aspx', 'jsp', 'rhtml', 'cgi',
+                'xhtml', 'jhtml', 'asp']
+
 GOOD_PATHS = ['story', 'article', 'feature', 'featured', 'slides',
               'slideshow', 'gallery', 'news', 'video', 'media',
               'v', 'radio', 'press']
@@ -86,6 +96,8 @@ def prepare_url(url, source_url=None):
 
 def valid_url(url, verbose=False, test=False):
     """
+    Is this URL a valid news-article url?
+
     Perform a regex check on an absolute url.
 
     First, perform a few basic checks like making sure the format of the url
@@ -116,10 +128,7 @@ def valid_url(url, verbose=False, test=False):
     We also filter out articles with a subdomain or first degree path
     on a registered bad keyword.
     """
-    DATE_REGEX = r'([\./\-_]{0,1}(19|20)\d{2})[\./\-_]{0,1}(([0-3]{0,1}[0-9][\./\-_])|(\w{3,5}[\./\-_]))([0-3]{0,1}[0-9][\./\-]{0,1})?'
-    ALLOWED_TYPES = ['html', 'htm', 'md', 'rst'] # TODO add more!
-
-    # if we are testing this method in the testing suite, we actually
+    # If we are testing this method in the testing suite, we actually
     # need to preprocess the url like we do in the article's constructor!
     if test:
         url = prepare_url(url)
@@ -151,15 +160,14 @@ def valid_url(url, verbose=False, test=False):
 
     # siphon out the file type. eg: .html, .htm, .md
     if len(path_chunks) > 0:
-
-        last_chunk = path_chunks[-1].split('.') # last chunk == file usually
-        file_type = last_chunk[-1] if len(last_chunk) >= 2 else None
+        file_type = url_to_filetype(url)
 
         # if the file type is a media type, reject instantly
         if file_type and file_type not in ALLOWED_TYPES:
             if verbose: print '\t%s rejected due to bad filetype' % url
             return False
 
+        last_chunk = path_chunks[-1].split('.')
         # the file type is not of use to use anymore, remove from url
         if len(last_chunk) > 1:
             path_chunks[-1] = last_chunk[-2]
@@ -179,7 +187,7 @@ def valid_url(url, verbose=False, test=False):
         if verbose: print '%s caught for a bad tld' % url
         return False
 
-    if  len(path_chunks) == 0:
+    if len(path_chunks) == 0:
         dash_count, underscore_count = 0, 0
     else:
         dash_count = url_slug.count('-')
@@ -217,9 +225,30 @@ def valid_url(url, verbose=False, test=False):
         if verbose: print '%s verified for date' % url
         return True
 
+    for GOOD in GOOD_PATHS:
+        if GOOD.lower() in [p.lower() for p in path_chunks]:
+            if verbose: print '%s verified for good path' % url
+            return True
+
     if verbose: print '%s caught for default false' % url
     return False
 
+def url_to_filetype(abs_url):
+    """
+    Input a URL and output the filetype of the file
+    specified by the url. Returns None for no filetype.
+    'http://blahblah/images/car.jpg' -> 'jpg'
+    'http://yahoo.com'               -> None
+    """
+    path = urlparse(abs_url).path
+    # Eliminate the trailing '/', we are extracting the file
+    if path.endswith('/'):
+        path = path[:-1]
+    path_chunks = [x for x in path.split('/') if len(x) > 0]
+    last_chunk = path_chunks[-1].split('.')  # last chunk == file usually
+    file_type = last_chunk[-1] if len(last_chunk) >= 2 else None
+    return file_type or None
+
 def get_domain(abs_url, **kwargs):
     """
     returns a url's domain, this method exists to
